package salestargets;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Map.Entry;

import javax.persistence.Persistence;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;

public class Main
{
	private static EntityManagerFactory emf = Persistence.createEntityManagerFactory("objectdb:/var/tmp/salestargets.tmp;drop");

	/**
	 * Create 'count' salesmen, and return a set of them
	 * 
	 * @param count
	 *            How many salesman instances to create
	 * @return Set of all salesmen created
	 */
	private static HashSet<Salesman> createSalesmen(int count)
	{
		EntityManager e = emf.createEntityManager();
		HashSet<Salesman> salesmen = new HashSet<Salesman>();
		e.getTransaction().begin();
		for (int i = 0; i < count; i++)
		{
			Salesman s = new Salesman((i + 1) + "");
			e.persist(s);
			salesmen.add(s);
		}
		e.getTransaction().commit();
		e.close();
		return salesmen;
	}

	/**
	 * For each salesman in salesmen, add companyCount companies, each with
	 * targetCount targets
	 * 
	 * @param salesmen
	 *            the salesman instances to modify
	 * @param companyCount
	 *            how many companies each salesman should have
	 * @param targetCount
	 *            how many targets each salesman should have for each company
	 */
	private static void addCompaniesToSalesmen(HashSet<Salesman> salesmen, int companyCount, int targetCount)
	{
		long totalTime = System.currentTimeMillis();
		for (int i = 0; i < companyCount; i++)
		{
			// There are good reasons for using a new EntityManager each loop,
			// as this
			// is just a contrived example
			EntityManager em = emf.createEntityManager();
			em.getTransaction().begin();
			Company company = new Company((i + 1) + "Company");
			em.persist(company);
			em.getTransaction().commit();
			 long time = System.currentTimeMillis();
			em.getTransaction().begin();
			 System.out.println("Company Before Merge:\t"+(System.currentTimeMillis()-time));
			for (Salesman s : salesmen)
				em.merge(s);
			 System.out.println("Company After Merge:\t"+(System.currentTimeMillis()-time));
			for (Salesman s : salesmen)
			{
				HashSet<Integer> tgts = new HashSet<Integer>();
				int multiple = s.getId();
				int start = multiple * 1000;
				for (int tgt = start; tgt < (start + (targetCount * multiple)); tgt += multiple)
				{
					tgts.add(tgt);
				}
				s.getCompanyTargets().put(company, tgts);
			}
			 System.out.println("Company Before commit:\t"+(System.currentTimeMillis()-time));
			em.getTransaction().commit();
			 System.out.println("Company After commit:\t"+(System.currentTimeMillis()-time));
			em.close();
			 System.out.println("Company After close:\t"+(System.currentTimeMillis()-time));
		}
		System.out.println("Company Total time:\t" + (System.currentTimeMillis() - totalTime));
	}

	/**
	 * Select a salesman and re-allocate all his company targets to other
	 * salesmen
	 * 
	 * @param salesmen
	 *            The set of salesmen
	 * @return Salesman The salesman instance which has had all its targets
	 *         removed
	 */
	private static Salesman redistributeTargets(HashSet<Salesman> salesmen)
	{
		EntityManager em = emf.createEntityManager();
		em.getTransaction().begin();
		long totalTime = System.currentTimeMillis();
		Salesman victim = salesmen.iterator().next();
		// System.out.println(victim + " is the victim");
		salesmen.remove(victim);
		Iterator<Salesman> salesmenIterator = salesmen.iterator();
		for (Entry<Company, HashSet<Integer>> entry : victim.getCompanyTargets().entrySet())
		{
			for (Integer tgt : entry.getValue())
			{
				Salesman nextSalesman = salesmenIterator.next();
				// System.out.println("Adding "+tgt+" for "+entry.getKey()+" to "+nextSalesman);
				nextSalesman.getCompanyTargets().get(entry.getKey()).add(tgt);
				if (!salesmenIterator.hasNext())
					salesmenIterator = salesmen.iterator();
			}
		}
		victim.getCompanyTargets().clear();
		System.out.println("Redist Before Merge:\t" + (System.currentTimeMillis() - totalTime));
		for (Salesman s : salesmen)
			em.merge(s);
		em.merge(victim);
		System.out.println("Redist After Merge:\t" + (System.currentTimeMillis() - totalTime));
		System.out.println("Redist Before Commit:\t" + (System.currentTimeMillis() - totalTime));
		em.getTransaction().commit();
		System.out.println("Redist Total Time:\t " + (System.currentTimeMillis() - totalTime));
		return victim;
	}

	public static void main(String[] args)
	{
		HashSet<Salesman> salesmen = createSalesmen(100);
		addCompaniesToSalesmen(salesmen, 100, 100);
		Salesman emptied = redistributeTargets(salesmen);
		System.out.println("Salesman " + emptied + " now has targets:" + emptied.getCompanyTargets());
	}
}
